#if defined (SR501_PIN)

//--------------------------------------------------------
struct struct_sr501{
  bool on_off;
  bool alarm;
  bool repeat;
  char cmd_on[CMD_LEN];       // Посылаемая команда при сработке
  char cmd_on_rep[CMD_LEN];   // Посылаемая команда во время сработки
  char cmd_off[CMD_LEN];      // Посылаемая команда после сработки
  char cmd_off_rep[CMD_LEN];  // Посылаемая команда во время пассива
  uint32_t time_from=0;
  uint32_t time_to=0;
  GHflags week=0;
  GHflags lamp=0;
  GHflags modul=0;
  GHflags timers=0;
  bool ok_on=true;
  bool ok_off=true;
};
struct_sr501 sr501[MAX_SR501];
FS_(sr501);

bool sr501_s;
//--------------------------------------------------------
void setup_SR501(){
  pinMode(SR501_PIN,INPUT);     
  FS_LOAD(sr501); 
  main_menu+=SR501_MENU;
  main_menu+=",";
}
//--------------------------------------------------------
void get_SR501() {
    char send_udp[25];
    Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
    sprintf_P(send_udp, (const char *)F("SR501 IS %s"), (sr501_s) ? "ON":"OFF");
    Udp.write(send_udp);
    Udp.endPacket();
}  
//--------------------------------------------------------
void SR501_send(uint8_t i, char cmd[]) {
  module_parsig(sr501[i].modul,cmd);
  IP_parsig(sr501[i].lamp,cmd);
}
//--------------------------------------------------------
void loop_SR501(){
  EVERY_MS(1000) {
  sr501_s=digitalRead(SR501_PIN);
  if (hub.focused()) hub.sendUpdate("led", String(sr501_s));  // обновляем светодиод датчика движения sr501
  FS_TICK(sr501); 
  FOR_i(0,MAX_SR501)
   if (sr501[i].on_off) 
     if (sr501[i].week.get(dayofweek)) {
          bool on_time=false;
          if (sr501[i].time_from<sr501[i].time_to) 
              on_time=(hour*60+minute>=round(sr501[i].time_from/60)) && (hour*60+minute<=round(sr501[i].time_to/60));
          else    
              on_time=(hour*60+minute>=round(sr501[i].time_from/60)) || (hour*60+minute<=round(sr501[i].time_to/60));

          if (on_time) {
             if (sr501_s) {
               if (sr501[i].ok_on) if (strlen(sr501[i].cmd_on)) { SR501_send(i,sr501[i].cmd_on); sr501[i].ok_on=false;}
               if (strlen(sr501[i].cmd_on_rep)) SR501_send(i,sr501[i].cmd_on_rep);
               sr501[i].ok_off=true;
             } else {
               if (sr501[i].ok_off) if (strlen(sr501[i].cmd_off)) { SR501_send(i,sr501[i].cmd_off); sr501[i].ok_off=false;}
               if (strlen(sr501[i].cmd_off_rep)) SR501_send(i,sr501[i].cmd_off_rep);
               sr501[i].ok_on=true;
             }
          } else {
             sr501[i].ok_on=true;
             sr501[i].ok_off=true;
          }
     }
  } //1000 мс
}
//--------------------------------------------------------
void SR501_parsing(char act_chr[]){
  
}
//--------------------------------------------------------
void HUB_module_SR501() {
  static uint8_t plus_minus[MAX_SR501];
  static uint8_t tab = 0;
  hub.WidgetSize(100);
  hub.LED_(F("led"), &sr501_s, F("Датчик движения"));
  
  FOR(tab,0,MAX_SR501) {
    hub.WidgetSize(40);
    eeprom_flag |= hub.Time(&sr501[tab].time_from, F("From"), sr501[tab].on_off ?  GH_GREEN : GH_YELLOW);
    eeprom_flag |= hub.Time(&sr501[tab].time_to, F("To"), sr501[tab].on_off ? GH_GREEN : GH_YELLOW);
    hub.WidgetSize(20);
    hub.Tabs(&plus_minus[tab]," - , + ");
    if (plus_minus[tab]) {
      hub.WidgetSize(100);
      eeprom_flag |= hub.Flags(&sr501[tab].week, str_dayofweek,F("Дни недели"));
      hub.WidgetSize(80);
      eeprom_flag |= hub.Input(&sr501[tab].cmd_on,      GH_CSTR, F("Команда при сработке"),HUB_LEN,"",GH_GREEN);
      hub.WidgetSize(20);
      eeprom_flag |= hub.Switch_("sr501_sw"+String(tab),&sr501[tab].on_off,F("Active"));
      hub.WidgetSize(80);
      eeprom_flag |= hub.Input(&sr501[tab].cmd_on_rep,  GH_CSTR, F("Команда во время сработки"),HUB_LEN,"",GH_GREEN);
      eeprom_flag |= hub.Input(&sr501[tab].cmd_off,     GH_CSTR, F("Команда после сработки"),HUB_LEN,"",GH_RED);
      eeprom_flag |= hub.Input(&sr501[tab].cmd_off_rep, GH_CSTR, F("Команда во время пассива"),HUB_LEN,"",GH_RED);
      hub.WidgetSize(100);
      eeprom_flag |= hub.Flags(&sr501[tab].lamp, str_GHflags_IP,F("Кому отсылаем"));
      eeprom_flag |= hub.Flags(&sr501[tab].modul, str_modul,F(hlp_modul));
    }
   }
   if (eeprom_flag) {
      FS_UPDATE(sr501);
      eeprom_flag=false;
   }
}

//--------------------------------------------------------
void load_SR501(){
   String CONFIG="";
   FOR_i(0,MAX_SR501) {
    CONFIG+="SR501;"+String(i);
    if (sr501[i].on_off) CONFIG+=";1;";
                      else CONFIG+=";0;";
    CONFIG+=String(sr501[i].alarm)+";";
    CONFIG+=String(sr501[i].time_from/3600)+":"+String((sr501[i].time_from/60)%60)+";";
    CONFIG+=String(sr501[i].time_to/3600)+":"+String((sr501[i].time_to/60)%60)+";";
    CONFIG+=sr501[i].week.toString()+";"+sr501[i].lamp.toString()+";"+sr501[i].modul.toString()+";";
    CONFIG+=String(sr501[i].cmd_on)+";";
    CONFIG+=String(sr501[i].cmd_on_rep)+";";
    CONFIG+=String(sr501[i].cmd_off)+";";
    CONFIG+=String(sr501[i].cmd_off_rep)+";"+char(10);
   }
   Udp.write(CONFIG.c_str());
   Udp.write(char(10));
}

#endif
